"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nock_1 = __importDefault(require("nock"));
const path_1 = __importDefault(require("path"));
const _1 = require(".");
const graphql_1 = require("graphql");
const introspectionResponse_json_1 = __importDefault(require("./__mocks__/introspectionResponse.json"));
describe('getDiff', () => {
    describe('remote schema fetching', () => {
        const testRemoteSchemaLocation = 'http://test/graphql';
        it('fetches remote schema successfully', async () => {
            nock_1.default(testRemoteSchemaLocation)
                .post('', JSON.stringify({ query: graphql_1.introspectionQuery }))
                .twice()
                .reply(200, introspectionResponse_json_1.default);
            const result = await _1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('throws error on status codes other than 200', () => {
            nock_1.default(testRemoteSchemaLocation)
                .post('', JSON.stringify({ query: graphql_1.introspectionQuery }))
                .twice()
                .reply(404);
            return expect(_1.getDiff(testRemoteSchemaLocation, testRemoteSchemaLocation)).rejects.toThrow(`404 - Not Found (${testRemoteSchemaLocation})`);
        });
        afterEach(() => {
            nock_1.default.cleanAll();
        });
    });
    describe('local schema reading', () => {
        it('works with exact path to file', async () => {
            const localSchemaLocation = path_1.default.join(__dirname, './__mocks__/localSchema.graphql');
            const result = await _1.getDiff(localSchemaLocation, localSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('works with glob pattern', async () => {
            const localSchemaLocation = path_1.default.join(__dirname, './__mocks__/**/localSchema.graphql');
            const result = await _1.getDiff(localSchemaLocation, localSchemaLocation);
            expect(result).toBeUndefined();
        });
        it('throws error on non-existent path', () => {
            return expect(_1.getDiff(path_1.default.join(__dirname, 'invalidLocation'), path_1.default.join(__dirname, 'invalidLocation'))).rejects.toThrow(/ENOENT/);
        });
        it('throws error on non-existent files in glob pattern', () => {
            return expect(_1.getDiff(path_1.default.join(__dirname, '/**/*.invalidgql'), path_1.default.join(__dirname, '/**/*.invalidgql'))).rejects.toThrow(/No types found with glob pattern '.*'/);
        });
    });
    describe('schema diffing', () => {
        it('returns the exact diff between two schemas', async () => {
            const result = await _1.getDiff(path_1.default.join(__dirname, '__mocks__/localSchema.graphql'), path_1.default.join(__dirname, '__mocks__/localSchemaDangerous.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.diff).toMatch(/\+  SECOND_VALUE/);
            }
        });
        it('returns dangerous changes', async () => {
            const result = await _1.getDiff(path_1.default.join(__dirname, '__mocks__/localSchema.graphql'), path_1.default.join(__dirname, '__mocks__/localSchemaDangerous.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.dangerousChanges).toEqual([
                    {
                        description: 'SECOND_VALUE was added to enum type TestEnum.',
                        type: 'VALUE_ADDED_TO_ENUM'
                    }
                ]);
            }
        });
        it('returns breaking changes', async () => {
            const result = await _1.getDiff(path_1.default.join(__dirname, '__mocks__/localSchema.graphql'), path_1.default.join(__dirname, '__mocks__/localSchemaBreaking.graphql'));
            expect(result).toBeDefined();
            if (result) {
                expect(result.breakingChanges).toEqual([
                    {
                        description: 'Query.test changed type from String to Int.',
                        type: 'FIELD_CHANGED_KIND'
                    }
                ]);
            }
        });
    });
});
